#|/usr/env python3
# -*- coding: utf8 -*-

import tkinter as tk
from tkinter import ttk
from tkinter import font
from datetime import datetime, date

COLORS = ['#eaecf0', '#99cbff', '#afeeee', '#b0ffb0', '#ccffcc', '#ffcccc']

class Mensile(tk.LabelFrame):
    ''' Visualizza le offerte presenti in un determinato mese.
        La visualizzazione viene effettuata tramite una tabella
        composta da tkinter.Label
    '''
    days_to_months = (31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31)
    months_names = ('Gennaio', 'Febbraio', 'Marzo', 'Aprile', 'Maggio',
                    'Giugno', 'Luglio', 'Agosto', 'Settembre', 'Ottobre',
                    'Novembre', 'Dicembre')
    def __init__(self, parent, data: list=None, *args, **kwargs):
        super().__init__(parent, *args, **kwargs)
        self.parent = parent
        self.year = datetime.today().year
        self.month = datetime.today().month
        self.days = 0
        self.data = data
        self.__resample()

    def set_mese(self, month: int) -> None:
        ''' Imposta il mese da visualizzare. '''
        # verifica il valore ricevuto
        if month < 1 or month > 12:
            raise ValueError('Mese non valido')
        if self.month == month: return
        self.month = month
        self.__resample()

    def __resample(self) -> None:
        for ch in self.winfo_children():
            ch.destroy()
        self.__make_headers()
        if self.data: self.__make_data()

    def __make_headers(self) -> None:
        # decide quanti giorni ha il mese
        if self.month == 2:
            try:
                val = date(self.year, 2, 29)
                self.days = 29
            except ValueError:
                self.days = 28
        else:
            self.days = self.days_to_months[self.month - 1]
        # crea le celle di intestazione
        lbl = tk.Label(self, relief='raised', text='Codice', bg='lightgray')
        lbl.grid(row=0, column=0, sticky='nsew')
        lbl = tk.Label(self, relief='raised', text='Descrizione', bg='lightgray')
        lbl.grid(row=0, column=1, sticky='nsew')
        lbl = tk.Label(self, relief='raised', text='prezzo', bg='lightgray')
        lbl.grid(row=0, column=2, sticky='nsew')
        for i in range(self.days):
            lbl = tk.Label(self, relief='raised', text=f'{i+1:02d}', bg='lightgray')
            lbl.grid(row=0, column=3+i, sticky='nsew')

    def __make_data(self) -> None:
        ''' Mostra gli articoli con promozioni ricadenti nel mese in esame. '''
        r_start = 1  # riga corrente di inserimento
        for art in self.data:
            # estrae le promozioni dell'articolo interessanti il mese
            promo = []
            for p in art['promozioni']:
                if (self.year == int(p['inizio'].split('-')[2]) or
                    self.year == int(p['fine'].split('-')[2])):
                    if (self.month >= int(p['inizio'].split('-')[1]) and
                        self.month <= int(p['fine'].split('-')[1])):
                        promo.append(p)
            if not promo:
                pass
            else:
                self.__make_art(art, promo, r_start)
                r_start += len(promo)

    def __make_art(self, art: dict, promo: list, row: int) -> None:
        ''' Mostra le promozioni di un singolo articolo. '''
        v_space = len(promo)  # calcolo righe necessarie
        # rappresentazione dati generali
        lbl = tk.Label(self, text=art['codice'], relief='sunken', justify='left')
        lbl.grid(row=row, column=0, rowspan=v_space, sticky='nsew')
        lbl = tk.Label(self, text=art['descrizione'], relief='sunken', justify='left')
        lbl.grid(row=row, column=1, rowspan=v_space, sticky='nsew')
        lbl = tk.Label(self, text=art['prezzo'], relief='sunken', justify='right')
        lbl.grid(row=row, column=2, rowspan=v_space, sticky='nsew')
        # calcola la dimensione del font corrente e la riduce di due punti
        fntsize = font.nametofont('TkTextFont').actual()['size'] - 2
        # rappresentazione singole offerte
        current = row
        color = 0
        for p in promo:
            i_day = 0
            e_day = 0
            # definizione didascalia AL MOMENTO NON UTILIZZATA
            txt = p['codice']
            if p['sc%']: txt += f' {p["sc%"]}%'
            if p['scv']: txt += f' € {p["scv"]}'
            txt += f' su € {p["prezzo"]}'
            # calcolo celle interessate dalla promozione
            if self.month != int(p['inizio'].split('-')[1]):
                i_day = 1
            elif self.month == int(p['inizio'].split('-')[1]):
                i_day = int(p['inizio'].split('-')[0])
            if self.month != int(p['fine'].split('-')[1]):
                e_day = self.days
            elif self.month == int(p['fine'].split('-')[1]):
                e_day = int(p['fine'].split('-')[0])
            # rappresentazione barra promozione
            span = e_day - i_day + 1
            lbl = tk.Label(self, text=txt,  # font=f'size={fntsize}',
                           relief='raised', bg=COLORS[color])
            lbl.grid(row=current, column=2+i_day, columnspan=span, sticky='nsew')
            self.update()
            current += 1
            color += 1

    @property
    def month_name(self) -> str:
        ''' Restituisce il nome del mese corrente. '''
        return self.months_names[self.month-1]

    @property
    def mont(self) -> int:
        return self.month

    def set_data(self, data: list) -> None:
        self.data = data
        self.__resample()
        
        
class Offerte(tk.Toplevel):
    ''' Classe per la visualizzazione delle offerte mensili. '''
    def __init__(self, parent: callable, *args, **kwargs):
        super().__init__(parent, *args, **kwargs)
        self.parent = parent
        self.pn_offerte = Mensile(self)
        self.pn_offerte.grid(row=0, column=0, sticky='nsew')
        self.grid_columnconfigure(0, weight=1)
        self.grid_rowconfigure(0, weight=1)
        pn_m = tk.Frame(self)
        pn_m.grid(row=1, column=0, padx=5, pady=5, sticky='ew')
        lbl = tk.Label(pn_m, text='Selezione mese:')
        lbl.grid(row=0, column=0, sticky='w')
        self.v_month = tk.IntVar()
        self.sp_mont = tk.Spinbox(pn_m, from_=1, to=12, textvariable=self.v_month,
                                  width=4, command=self.__on_month)
        self.sp_mont.grid(row=0, column=1, padx=5, pady=5, sticky='w')
        self.after_idle(self.__set_current)
        self.update()
        

    def set_data(self, data: list) -> None:
        self.pn_offerte.set_data(data)
        self.update()

    def __on_month(self) -> None:
        month = self.v_month.get()
        self.pn_offerte.set_mese(month)
        self.after_idle(self.__set_current)

    def __set_current(self):
        self.title(f'Offerte mese di {self.pn_offerte.month_name}')
        self.v_month.set(self.pn_offerte.month)
        self.update()


class ElArticoli(ttk.Treeview):
    def __init__(self, data: list, parent: callable, *args, **kwargs):
        super().__init__(parent, *args, **kwargs)
        self.parent = parent
        self.articoli = data
        self.selected = []
        #self.configure(selectmode='browse')
        didas = ('Codice', 'Marchio', 'Modello', 'Descrizione',
                 'Quantità', 'Prezzo', 'Offerte')
        self['columns'] = didas
        self['show'] = 'headings'
        for t in didas:
            self.heading(t, text=t)
        self.column('Codice', width=100, stretch=False)
        self.column('Marchio', width=100, stretch=False)
        self.column('Modello', width=100, stretch=False)
        self.column('Descrizione', minwidth=100, stretch=True)
        self.column('Quantità', width=75, anchor=tk.E, stretch=False)
        self.column('Prezzo', width=75, anchor=tk.E, stretch=False)
        self.column('Offerte', width=75, anchor=tk.E, stretch=False)
        self.__show_data()
        self.bind("<<TreeviewSelect>>", self.__on_select)

    def __show_data(self) -> None:
        for a in self.articoli:
            row = [a['codice'], a['marchio'], a['modello'], a['descrizione'],
                   a['quantita'], a['prezzo'], f'{len(a["promozioni"])}']
            self.insert('', 'end', iid=None, values=row)

    def __on_select(self, evt: callable) -> None:
        self.selected = []
        w = evt.widget
        sel = w.selection()
        data = []
        for iid in sel:
            idx = self.index(iid)
            data.append(self.articoli[idx])
        self.selectd = data
        self.parent.set_articolo(self.selectd[0])
        self.parent.set_selection(self.selectd)

    def get_selection(self) -> list:
        if not self.selected: return
        self.parent.set_selection(self.selectd)
    
        
class ElPromozioni(ttk.Treeview):
    def __init__(self, parent: callable, data: list=None, *args, **kwargs):
        super().__init__(parent, *args, **kwargs)
        self.parent = parent
        self.articolo = data
        self.configure(selectmode='browse')
        didas = ('Codice', 'Attiva', 'Cum.', 'Sogg.', 'Fornitore',
                 'Inizio', 'Fine', 'Sc %', 'Sc €', 'Prezzo')
        self['columns'] = didas
        self['show'] = 'headings'
        for t in didas:
            self.heading(t, text=t)
        self.column('Codice', width=90, stretch=False)
        self.column('Attiva', width=70, stretch=False)
        self.column('Cum.', width=50, stretch=False)
        self.column('Sogg.', width=50, stretch=False)
        self.column('Fornitore', width=120, stretch=False)
        self.column('Inizio', width=100, anchor=tk.CENTER, stretch=False)
        self.column('Fine', width=100, anchor=tk.CENTER, stretch=False)
        self.column('Sc %', width=100, anchor=tk.E, stretch=False)
        self.column('Sc €', width=100, anchor=tk.E, stretch=False)
        self.column('Prezzo', width=100, anchor=tk.E, stretch=False)
        self.__show_data()
    
    def __show_data(self) -> None:
        if not self.articolo: return
        sort_prom = self.articolo['promozioni'].copy()
        sort_prom.sort(key=lambda x: ''.join(x['fine'].split('-')[::-1]), reverse=True)
        for p in sort_prom:
            row = [p['codice'], p['attivo'], p['cumulabile'], p['soggetto'],
                   p['fornitore'], p['inizio'], p['fine'], p['sc%'],
                   p['scv'], p['prezzo']]
            self.insert('', 'end', iid=None, values=row)

    def set_article(self, art: dict) -> None:
        ''' Imposta l'articolo corrente. '''
        self.delete(*self.get_children())
        self.articolo = art
        self.__show_data()
        
    
    
class App(tk.Tk):
    ''' Maschera di visualizzazione articoli - finestra principale. '''
    def __init__(self):
        super().__init__()
        self.articoli = []
        self.single = True
        self.win_prom = None
        self.__get_data()
        self.__populate()

    def __get_data(self):
        ''' Legge i dati e crea una lista di dizionari degli articoli
            oltre strutture dati di utilità associate.
        '''
        fname = 'File_Prova.csv'
        with open(fname) as f:
            data = [x.split(';') for x in f.readlines() if x]
        articles = []
        codes = set([x[8] for x in data[1:]])
        for c in codes:
            values = [x for x in data if x[8] == c]
            if not values:
                pass
            else:
                art = {}
                art['codice'] = values[0][8]
                art['modello'] = values[0][9]
                art['marchio'] = values[0][10]
                art['descrizione'] = values[0][11]
                art['quantita'] = values[0][12]
                art['prezzo'] = f'{max([float(x[15]) for x in values]):.2f}'
                art['promozioni'] = []                
                for d in values:
                    prom = {}
                    prom['codice'] = d[0]
                    prom['competenza'] = d[1]
                    prom['attivo'] = d[2]
                    prom['cumulabile'] = d[3]
                    prom['soggetto'] = d[4]
                    prom['fornitore'] = d[5]
                    prom['inizio'] = d[6]
                    prom['fine'] = d[7]
                    prom['sc%'] = d[13]
                    prom['scv'] = d[14]
                    prom['prezzo'] = f'{float(d[15]):0.2f}'
                    art['promozioni'].append(prom)
                articles.append(art)
        articles.sort(key=lambda x: x['codice'])
        self.articoli = articles

    def __populate(self) -> None:
        lbl = tk.Label(self, text='Elenco Articoli:', font='bold')
        lbl.grid(row=0, column=0, padx=5, pady=(5,2), sticky='w')
        pn_data = tk.Frame(self)
        pn_data.set_articolo = self.set_articolo
        pn_data.set_selection = self.set_selection
        pn_data.grid(row=1, column=0, padx=5, pady=(2,5), sticky='nsew')
        self.t_art = ElArticoli(self.articoli, pn_data)
        self.t_art.grid(row=0, column=0, padx=(5,1), pady=(1,1), sticky='nsew')
        vscr = tk.Scrollbar(pn_data, orient=tk.VERTICAL, command=self.t_art.yview)
        self.t_art.configure(yscrollcommand=vscr.set)
        vscr.grid(row=0, column=1, padx=(1,5), pady=(5,1), sticky='ns')
        hscr = tk.Scrollbar(pn_data, orient=tk.HORIZONTAL, command=self.t_art.xview)
        self.t_art.configure(xscrollcommand=hscr.set)
        hscr.grid(row=1, column=0, padx=(1,5), pady=(1,5), sticky='ew')
        pn_data.grid_rowconfigure(0, weight=1)
        pn_data.grid_columnconfigure(0, weight=1)
        lbl = tk.Label(self, text="Promozioni dell'articolo:")
        lbl.grid(row=2, column=0, padx=5, pady=(5,2), sticky='w')
        pn_prom = tk.Frame(self)
        pn_prom.grid(row=3, column=0, padx=5, pady=(2,5), sticky='nsew')
        self.t_promo = ElPromozioni(pn_prom, height=3)
        self.t_promo.grid(row=0, column=0, padx=(5,1), pady=(1,1), sticky='nsew')
        vscr = tk.Scrollbar(pn_prom, orient=tk.VERTICAL, command=self.t_promo.yview)
        self.t_promo.configure(yscrollcommand=vscr.set)
        vscr.grid(row=0, column=1, padx=(1,5), pady=(5,1), sticky='ns')
        hscr = tk.Scrollbar(pn_prom, orient=tk.HORIZONTAL, command=self.t_promo.xview)
        self.t_promo.configure(xscrollcommand=hscr.set)
        hscr.grid(row=1, column=0, padx=(1,5), pady=(1,5), sticky='ew')
        pn_prom.grid_rowconfigure(0, weight=1)
        pn_prom.grid_columnconfigure(0, weight=1)
        pn_cmd = tk.Frame(self)
        pn_cmd.grid(row=4, column=0, padx=5, pady=5, sticky='ew')
        bt_single = tk.Button(pn_cmd, text='Calendario singolo articolo',
                              command=self.__on_single)
        bt_single.grid(row=0, column=0, padx=2, pady=2, sticky='ew')
        bt_global = tk.Button(pn_cmd, text='Calendario articoli multipli',
                              command=self.__on_select)
        bt_global.grid(row=0, column=1, padx=2, pady=2, sticky='ew')
        bt_end = tk.Button(pn_cmd, text='Chiudi',
                              command=self.destroy)
        bt_end.grid(row=0, column=2, padx=2, pady=2, sticky='ew')
        for i in range(3): pn_cmd.grid_columnconfigure(i, weight=1, uniform='bt')
        

        self.grid_rowconfigure(1, weight=1)
        self.grid_columnconfigure(0, weight=1)

    def set_articolo(self, idx: dict) -> None:
        self.t_promo.set_article(idx)

    def __on_single(self) -> None:
        self.single = True        
        if not self.win_prom:
            self.win_prom = Offerte(self)
            self.win_prom.mainloop()
        
    def __on_select(self) -> None:
        self.single = False
        if not self.win_prom:
            self.win_prom = Offerte(self)
            self.win_prom.mainloop()

    def set_selection(self, data: list) -> None:
        if not self.win_prom: return
        if self.single:
            self.win_prom.set_data([data[0],])
        else:
            self.win_prom.set_data(data)

    def __on_close(self) -> None:
        if self.win_prom:
            self.win_prom.destroy()
            self.win_prom = None
        self.destroy



if __name__ == '__main__':
    app = App()
    app.mainloop()
        
        
